# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .odds_format import OddsFormat
from .parlay_leg_request import ParlayLegRequest


class RoundRobinOptions(Enum):
    """An enumeration representing different categories.

    :cvar PARLAY: "Parlay"
    :vartype PARLAY: str
    :cvar TWOLEGROUNDROBIN: "TwoLegRoundRobin"
    :vartype TWOLEGROUNDROBIN: str
    :cvar THREELEGROUNDROBIN: "ThreeLegRoundRobin"
    :vartype THREELEGROUNDROBIN: str
    :cvar FOURLEGROUNDROBIN: "FourLegRoundRobin"
    :vartype FOURLEGROUNDROBIN: str
    :cvar FIVELEGROUNDROBIN: "FiveLegRoundRobin"
    :vartype FIVELEGROUNDROBIN: str
    :cvar SIXLEGROUNDROBIN: "SixLegRoundRobin"
    :vartype SIXLEGROUNDROBIN: str
    :cvar SEVENLEGROUNDROBIN: "SevenLegRoundRobin"
    :vartype SEVENLEGROUNDROBIN: str
    :cvar EIGHTLEGROUNDROBIN: "EightLegRoundRobin"
    :vartype EIGHTLEGROUNDROBIN: str
    """

    PARLAY = "Parlay"
    TWOLEGROUNDROBIN = "TwoLegRoundRobin"
    THREELEGROUNDROBIN = "ThreeLegRoundRobin"
    FOURLEGROUNDROBIN = "FourLegRoundRobin"
    FIVELEGROUNDROBIN = "FiveLegRoundRobin"
    SIXLEGROUNDROBIN = "SixLegRoundRobin"
    SEVENLEGROUNDROBIN = "SevenLegRoundRobin"
    EIGHTLEGROUNDROBIN = "EightLegRoundRobin"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, RoundRobinOptions._member_map_.values()))


@JsonMap(
    {
        "unique_request_id": "uniqueRequestId",
        "accept_better_line": "acceptBetterLine",
        "risk_amount": "riskAmount",
        "odds_format": "oddsFormat",
        "round_robin_options": "roundRobinOptions",
    }
)
class PlaceParlayBetRequest(BaseModel):
    """Request to place a bet

    :param unique_request_id: This unique id of the place bet requests. This is to support idempotent requests., defaults to None
    :type unique_request_id: str, optional
    :param accept_better_line: Whether or not to accept a bet when there is a line change in favor of the client., defaults to None
    :type accept_better_line: bool, optional
    :param risk_amount: Amount in client’s currency. It is always risk amount when placing Parlay bets NOTE: If round robin options is used this amount will apply for all parlays so actual amount wagered will be riskAmount X number of Parlays., defaults to None
    :type risk_amount: float, optional
    :param odds_format: Bet odds format.  <br>AMERICAN = American odds format,  <br>DECIMAL = Decimal (European) odds format,  <br>HONGKONG = Hong Kong odds format,  <br>INDONESIAN = Indonesian odds format,  <br>MALAY = Malaysian odds format <br>, defaults to None
    :type odds_format: OddsFormat, optional
    :param legs: Collection of parlay legs., defaults to None
    :type legs: List[ParlayLegRequest], optional
    :param round_robin_options: round_robin_options, defaults to None
    :type round_robin_options: List[RoundRobinOptions], optional
    """

    def __init__(
        self,
        unique_request_id: str = None,
        accept_better_line: bool = None,
        risk_amount: float = None,
        odds_format: OddsFormat = None,
        legs: List[ParlayLegRequest] = None,
        round_robin_options: List[RoundRobinOptions] = None,
    ):
        if unique_request_id is not None:
            self.unique_request_id = unique_request_id
        if accept_better_line is not None:
            self.accept_better_line = accept_better_line
        if risk_amount is not None:
            self.risk_amount = risk_amount
        if odds_format is not None:
            self.odds_format = self._enum_matching(
                odds_format, OddsFormat.list(), "odds_format"
            )
        if legs is not None:
            self.legs = self._define_list(legs, ParlayLegRequest)
        if round_robin_options is not None:
            self.round_robin_options = self._define_list(
                round_robin_options, RoundRobinOptions
            )



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.place_teaser_bet_response import PlaceTeaserBetResponse
from ..models.place_teaser_bet_request import PlaceTeaserBetRequest
from ..models.place_straight_bet_response import PlaceStraightBetResponse
from ..models.place_parlay_bet_response import PlaceParlayBetResponse
from ..models.place_parlay_bet_request import PlaceParlayBetRequest
from ..models.place_bet_request import PlaceBetRequest
from ..models.multi_bet_response_special_bet_response import (
    MultiBetResponseSpecialBetResponse,
)
from ..models.multi_bet_request_special_bet_request import (
    MultiBetRequestSpecialBetRequest,
)


class PlaceBetsService(BaseService):

    @cast_models
    def bets_straight_v2(
        self, request_body: PlaceBetRequest
    ) -> PlaceStraightBetResponse:
        """Place straight bet (SPREAD, MONEYLINE, TOTAL_POINTS, TEAM_TOTAL_POINTS).

        Please note when the status is PENDING_ACCEPTANCE and if the live delay was applied, the response will not have betId. Client would have to call /bets by uniqueRequestId to check the status if the bet was ACCEPTED.  For more details please see [How to place a bet on live events?](https://github.com/pinnacleapi/pinnacleapi-documentation/blob/master/FAQ.md#how-to-place-a-bet-on-live-events)

        Make sure you handle properly the case of an unexpected error as per
         [How to handle error on placing a bet?](https://github.com/pinnacleapi/pinnacleapi-documentation/blob/master/FAQ.md#how-to-handle-unexpected-error-on-placing-a-bet)

        :param request_body: The request body.
        :type request_body: PlaceBetRequest
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: OK
        :rtype: PlaceStraightBetResponse
        """

        Validator(PlaceBetRequest).validate(request_body)

        serialized_request = (
            Serializer(f"{self.base_url}/v4/bets/straight", self.get_default_headers())
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return PlaceStraightBetResponse._unmap(response)

    @cast_models
    def bets_parlay_v2(
        self, request_body: PlaceParlayBetRequest
    ) -> PlaceParlayBetResponse:
        """Make sure you handle properly the case of an unexpected error as per
         [How to handle error on placing a bet?](https://github.com/pinnacleapi/pinnacleapi-documentation/blob/master/FAQ.md#how-to-handle-unexpected-error-on-placing-a-bet)

        :param request_body: The request body.
        :type request_body: PlaceParlayBetRequest
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: OK
        :rtype: PlaceParlayBetResponse
        """

        Validator(PlaceParlayBetRequest).validate(request_body)

        serialized_request = (
            Serializer(f"{self.base_url}/v4/bets/parlay", self.get_default_headers())
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return PlaceParlayBetResponse._unmap(response)

    @cast_models
    def bets_teaser(
        self, request_body: PlaceTeaserBetRequest
    ) -> PlaceTeaserBetResponse:
        """Make sure you handle properly the case of an unexpected error as per
         [How to handle error on placing a bet?](https://github.com/pinnacleapi/pinnacleapi-documentation/blob/master/FAQ.md#how-to-handle-unexpected-error-on-placing-a-bet)

        :param request_body: The request body.
        :type request_body: PlaceTeaserBetRequest
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: OK
        :rtype: PlaceTeaserBetResponse
        """

        Validator(PlaceTeaserBetRequest).validate(request_body)

        serialized_request = (
            Serializer(f"{self.base_url}/v4/bets/teaser", self.get_default_headers())
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return PlaceTeaserBetResponse._unmap(response)

    @cast_models
    def bets_special(
        self, request_body: MultiBetRequestSpecialBetRequest
    ) -> MultiBetResponseSpecialBetResponse:
        """Make sure you handle properly the case of an unexpected error as per
         [How to handle error on placing a bet?](https://github.com/pinnacleapi/pinnacleapi-documentation/blob/master/FAQ.md#how-to-handle-unexpected-error-on-placing-a-bet)

        :param request_body: The request body.
        :type request_body: MultiBetRequestSpecialBetRequest
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: OK
        :rtype: MultiBetResponseSpecialBetResponse
        """

        Validator(MultiBetRequestSpecialBetRequest).validate(request_body)

        serialized_request = (
            Serializer(f"{self.base_url}/v4/bets/special", self.get_default_headers())
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return MultiBetResponseSpecialBetResponse._unmap(response)



# This file was generated by liblab | https://liblab.com/

from enum import Enum
from .utils.json_map import JsonMap
from .base import BaseModel


class RoundRobinOption(Enum):
    """An enumeration representing different categories.

    :cvar PARLAY: "Parlay"
    :vartype PARLAY: str
    :cvar TWOLEGROUNDROBIN: "TwoLegRoundRobin"
    :vartype TWOLEGROUNDROBIN: str
    :cvar THREELEGROUNDROBIN: "ThreeLegRoundRobin"
    :vartype THREELEGROUNDROBIN: str
    :cvar FOURLEGROUNDROBIN: "FourLegRoundRobin"
    :vartype FOURLEGROUNDROBIN: str
    :cvar FIVELEGROUNDROBIN: "FiveLegRoundRobin"
    :vartype FIVELEGROUNDROBIN: str
    :cvar SIXLEGROUNDROBIN: "SixLegRoundRobin"
    :vartype SIXLEGROUNDROBIN: str
    :cvar SEVENLEGROUNDROBIN: "SevenLegRoundRobin"
    :vartype SEVENLEGROUNDROBIN: str
    :cvar EIGHTLEGROUNDROBIN: "EightLegRoundRobin"
    :vartype EIGHTLEGROUNDROBIN: str
    """

    PARLAY = "Parlay"
    TWOLEGROUNDROBIN = "TwoLegRoundRobin"
    THREELEGROUNDROBIN = "ThreeLegRoundRobin"
    FOURLEGROUNDROBIN = "FourLegRoundRobin"
    FIVELEGROUNDROBIN = "FiveLegRoundRobin"
    SIXLEGROUNDROBIN = "SixLegRoundRobin"
    SEVENLEGROUNDROBIN = "SevenLegRoundRobin"
    EIGHTLEGROUNDROBIN = "EightLegRoundRobin"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, RoundRobinOption._member_map_.values()))


@JsonMap(
    {
        "round_robin_option": "roundRobinOption",
        "unrounded_decimal_odds": "unroundedDecimalOdds",
    }
)
class RoundRobinOptionWithOdds(BaseModel):
    """RoundRobinOptionWithOdds

    :param round_robin_option: round_robin_option, defaults to None
    :type round_robin_option: RoundRobinOption, optional
    :param odds: Parlay odds for this option., defaults to None
    :type odds: float, optional
    :param unrounded_decimal_odds: Unrounded parlay odds in decimal format to be used for calculations only, defaults to None
    :type unrounded_decimal_odds: float, optional
    """

    def __init__(
        self,
        round_robin_option: RoundRobinOption = None,
        odds: float = None,
        unrounded_decimal_odds: float = None,
    ):
        if round_robin_option is not None:
            self.round_robin_option = self._enum_matching(
                round_robin_option, RoundRobinOption.list(), "round_robin_option"
            )
        if odds is not None:
            self.odds = odds
        if unrounded_decimal_odds is not None:
            self.unrounded_decimal_odds = unrounded_decimal_odds



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class OddsFormat(Enum):
    """An enumeration representing different categories.

    :cvar AMERICAN: "AMERICAN"
    :vartype AMERICAN: str
    :cvar DECIMAL: "DECIMAL"
    :vartype DECIMAL: str
    :cvar HONGKONG: "HONGKONG"
    :vartype HONGKONG: str
    :cvar INDONESIAN: "INDONESIAN"
    :vartype INDONESIAN: str
    :cvar MALAY: "MALAY"
    :vartype MALAY: str
    """

    AMERICAN = "AMERICAN"
    DECIMAL = "DECIMAL"
    HONGKONG = "HONGKONG"
    INDONESIAN = "INDONESIAN"
    MALAY = "MALAY"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, OddsFormat._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from typing import Any, Dict


class BaseHeader:
    def set_value(self, value: Any) -> None:
        pass

    def get_headers(self) -> Dict[str, str]:
        pass



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .place_teaser_bet_leg_response import PlaceTeaserBetLegResponse


class PlaceTeaserBetResponseErrorCode(Enum):
    """An enumeration representing different categories.

    :cvar ABOVE_MAX_BET_AMOUNT: "ABOVE_MAX_BET_AMOUNT"
    :vartype ABOVE_MAX_BET_AMOUNT: str
    :cvar ALL_BETTING_CLOSED: "ALL_BETTING_CLOSED"
    :vartype ALL_BETTING_CLOSED: str
    :cvar BELOW_MIN_BET_AMOUNT: "BELOW_MIN_BET_AMOUNT"
    :vartype BELOW_MIN_BET_AMOUNT: str
    :cvar BLOCKED_BETTING: "BLOCKED_BETTING"
    :vartype BLOCKED_BETTING: str
    :cvar BLOCKED_CLIENT: "BLOCKED_CLIENT"
    :vartype BLOCKED_CLIENT: str
    :cvar DOUBLE_HIT: "DOUBLE_HIT"
    :vartype DOUBLE_HIT: str
    :cvar DUPLICATE_CLIENT_REFERENCE_ID: "DUPLICATE_CLIENT_REFERENCE_ID"
    :vartype DUPLICATE_CLIENT_REFERENCE_ID: str
    :cvar INCOMPLETE_CUSTOMER_BETTING_PROFILE: "INCOMPLETE_CUSTOMER_BETTING_PROFILE"
    :vartype INCOMPLETE_CUSTOMER_BETTING_PROFILE: str
    :cvar INSUFFICIENT_FUNDS: "INSUFFICIENT_FUNDS"
    :vartype INSUFFICIENT_FUNDS: str
    :cvar INVALID_COUNTRY: "INVALID_COUNTRY"
    :vartype INVALID_COUNTRY: str
    :cvar INVALID_CUSTOMER_PROFILE: "INVALID_CUSTOMER_PROFILE"
    :vartype INVALID_CUSTOMER_PROFILE: str
    :cvar INVALID_LEGS: "INVALID_LEGS"
    :vartype INVALID_LEGS: str
    :cvar INVALID_REQUEST: "INVALID_REQUEST"
    :vartype INVALID_REQUEST: str
    :cvar ODDS_FORMAT_MISMATCH: "ODDS_FORMAT_MISMATCH"
    :vartype ODDS_FORMAT_MISMATCH: str
    :cvar TEASER_DOES_NOT_EXIST: "TEASER_DOES_NOT_EXIST"
    :vartype TEASER_DOES_NOT_EXIST: str
    :cvar SAME_EVENT_ONLY_REQUIRED: "SAME_EVENT_ONLY_REQUIRED"
    :vartype SAME_EVENT_ONLY_REQUIRED: str
    :cvar SYSTEM_ERROR_1: "SYSTEM_ERROR_1"
    :vartype SYSTEM_ERROR_1: str
    :cvar SYSTEM_ERROR_2: "SYSTEM_ERROR_2"
    :vartype SYSTEM_ERROR_2: str
    :cvar SYSTEM_ERROR_3: "SYSTEM_ERROR_3"
    :vartype SYSTEM_ERROR_3: str
    :cvar TOO_FEW_LEGS: "TOO_FEW_LEGS"
    :vartype TOO_FEW_LEGS: str
    :cvar TOO_MANY_LEGS: "TOO_MANY_LEGS"
    :vartype TOO_MANY_LEGS: str
    :cvar DUPLICATED_REQUEST: "DUPLICATED_REQUEST"
    :vartype DUPLICATED_REQUEST: str
    :cvar RESPONSIBLE_BETTING_LOSS_LIMIT_EXCEEDED: "RESPONSIBLE_BETTING_LOSS_LIMIT_EXCEEDED"
    :vartype RESPONSIBLE_BETTING_LOSS_LIMIT_EXCEEDED: str
    :cvar RESPONSIBLE_BETTING_RISK_LIMIT_EXCEEDED: "RESPONSIBLE_BETTING_RISK_LIMIT_EXCEEDED"
    :vartype RESPONSIBLE_BETTING_RISK_LIMIT_EXCEEDED: str
    """

    ABOVE_MAX_BET_AMOUNT = "ABOVE_MAX_BET_AMOUNT"
    ALL_BETTING_CLOSED = "ALL_BETTING_CLOSED"
    BELOW_MIN_BET_AMOUNT = "BELOW_MIN_BET_AMOUNT"
    BLOCKED_BETTING = "BLOCKED_BETTING"
    BLOCKED_CLIENT = "BLOCKED_CLIENT"
    DOUBLE_HIT = "DOUBLE_HIT"
    DUPLICATE_CLIENT_REFERENCE_ID = "DUPLICATE_CLIENT_REFERENCE_ID"
    INCOMPLETE_CUSTOMER_BETTING_PROFILE = "INCOMPLETE_CUSTOMER_BETTING_PROFILE"
    INSUFFICIENT_FUNDS = "INSUFFICIENT_FUNDS"
    INVALID_COUNTRY = "INVALID_COUNTRY"
    INVALID_CUSTOMER_PROFILE = "INVALID_CUSTOMER_PROFILE"
    INVALID_LEGS = "INVALID_LEGS"
    INVALID_REQUEST = "INVALID_REQUEST"
    ODDS_FORMAT_MISMATCH = "ODDS_FORMAT_MISMATCH"
    TEASER_DOES_NOT_EXIST = "TEASER_DOES_NOT_EXIST"
    SAME_EVENT_ONLY_REQUIRED = "SAME_EVENT_ONLY_REQUIRED"
    SYSTEM_ERROR_1 = "SYSTEM_ERROR_1"
    SYSTEM_ERROR_2 = "SYSTEM_ERROR_2"
    SYSTEM_ERROR_3 = "SYSTEM_ERROR_3"
    TOO_FEW_LEGS = "TOO_FEW_LEGS"
    TOO_MANY_LEGS = "TOO_MANY_LEGS"
    DUPLICATED_REQUEST = "DUPLICATED_REQUEST"
    RESPONSIBLE_BETTING_LOSS_LIMIT_EXCEEDED = "RESPONSIBLE_BETTING_LOSS_LIMIT_EXCEEDED"
    RESPONSIBLE_BETTING_RISK_LIMIT_EXCEEDED = "RESPONSIBLE_BETTING_RISK_LIMIT_EXCEEDED"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value, PlaceTeaserBetResponseErrorCode._member_map_.values()
            )
        )


@JsonMap(
    {
        "error_code": "errorCode",
        "bet_id": "betId",
        "unique_request_id": "uniqueRequestId",
        "invalid_legs": "invalidLegs",
        "valid_legs": "validLegs",
    }
)
class PlaceTeaserBetResponse(BaseModel):
    """PlaceTeaserBetResponse

    :param status: Status of the request., defaults to None
    :type status: str, optional
    :param error_code: When Status is PROCESSED_WITH_ERROR, provides a code indicating the specific problem.<br><br>ABOVE_MAX_BET_AMOUNT = Bet is above the maximum allowed,  <br>ALL_BETTING_CLOSED = The wagering is disabled in the system (not related to a customer),  <br>BELOW_MIN_BET_AMOUNT = Bet is below the minimum allowed,  <br>BLOCKED_BETTING = Betting is suspended for the client,  <br>BLOCKED_CLIENT = Customer is inactive in the system,  <br>DOUBLE_HIT = The website submitted the same bet more than once,  <br>DUPLICATE_CLIENT_REFERENCE_ID = The teaser unique id and/or one of the leg unique id are the same,  <br>INCOMPLETE_CUSTOMER_BETTING_PROFILE = The customer does not exist,  <br>INSUFFICIENT_FUNDS = The risk amount is above the customer’s available balance,  <br>INVALID_COUNTRY = Current location is proscribed,  <br>INVALID_CUSTOMER_PROFILE = Either the customer does not exist OR the customer business rules are not verified,  <br>INVALID_LEGS = One or more legs are not verified,  <br>INVALID_REQUEST = Teaser request is not valid,  <br>ODDS_FORMAT_MISMATCH = Agent customer’s odds format differs from wager request odds format,<br>TEASER_DOES_NOT_EXIST = Teaser does not exist in the system,  <br>SAME_EVENT_ONLY_REQUIRED = Legs required to be for the same game only. Specified in the Teaser Specifications,  <br>SYSTEM_ERROR_1 = System error,  <br>SYSTEM_ERROR_2 = System error,  <br>SYSTEM_ERROR_3 = System error,  <br>TOO_FEW_LEGS = Legs count is below Min Picks specified in the Teaser Specifications,  <br>TOO_MANY_LEGS = Legs count is above Max Picks specified in the Teaser Specifications,<br>DUPLICATED_REQUEST = Request with the same uniqueRequestId was already processed. Please set the new value if you still want the request to be processed,  <br>RESPONSIBLE_BETTING_LOSS_LIMIT_EXCEEDED = Client has reached his total loss limit,  <br>RESPONSIBLE_BETTING_RISK_LIMIT_EXCEEDED = Client has reached his total risk limit<br>, defaults to None
    :type error_code: PlaceTeaserBetResponseErrorCode, optional
    :param bet_id: Id of a newly created bet., defaults to None
    :type bet_id: int, optional
    :param unique_request_id: Unique identifier provided in the request., defaults to None
    :type unique_request_id: str, optional
    :param price: Price for the bet., defaults to None
    :type price: float, optional
    :param risk: Amount wagered., defaults to None
    :type risk: float, optional
    :param win: Potential winnings., defaults to None
    :type win: float, optional
    :param invalid_legs: A collection of invalid legs, if any., defaults to None
    :type invalid_legs: List[PlaceTeaserBetLegResponse], optional
    :param valid_legs: A collection of valid legs, if any., defaults to None
    :type valid_legs: List[PlaceTeaserBetLegResponse], optional
    """

    def __init__(
        self,
        status: str = None,
        error_code: PlaceTeaserBetResponseErrorCode = None,
        bet_id: int = None,
        unique_request_id: str = None,
        price: float = None,
        risk: float = None,
        win: float = None,
        invalid_legs: List[PlaceTeaserBetLegResponse] = None,
        valid_legs: List[PlaceTeaserBetLegResponse] = None,
    ):
        if status is not None:
            self.status = status
        if error_code is not None:
            self.error_code = self._enum_matching(
                error_code, PlaceTeaserBetResponseErrorCode.list(), "error_code"
            )
        if bet_id is not None:
            self.bet_id = bet_id
        if unique_request_id is not None:
            self.unique_request_id = unique_request_id
        if price is not None:
            self.price = price
        if risk is not None:
            self.risk = risk
        if win is not None:
            self.win = win
        if invalid_legs is not None:
            self.invalid_legs = self._define_list(
                invalid_legs, PlaceTeaserBetLegResponse
            )
        if valid_legs is not None:
            self.valid_legs = self._define_list(valid_legs, PlaceTeaserBetLegResponse)
